@@ -7,7 +7,7 @@
  *
  *       http://people.redhat.com/drepper/sha-crypt.html
  *
- *  GPLv2 only - Copyright (C) 2008 - 2010
+ *  GPLv2 only - Copyright (C) 2008 - 2012
  *               David Sommerseth <dazo@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or
@@ -45,12 +45,17 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdint.h>
-#include <endian.h>
+#ifdef HAVE_SYS_ENDIAN_H
+# include <sys/endian.h>
+#else
+# include <endian.h>
+#endif /* HAVE_SYS_ENDIAN_H */
 #include <errno.h>
 #include <limits.h>
 #include <stdbool.h>
 #include <sys/param.h>
 #include <sys/types.h>
+#include <sys/mman.h>
 #include <assert.h>
 #include <time.h>
 #include <unistd.h>
@@ -326,7 +331,8 @@ inline char *sha512_crypt_r(const char *key, const char *salt, size_t maxrounds_
         /* Create byte sequence P. */
         cp = p_bytes = alloca (key_len);
         for (cnt = key_len; cnt >= 64; cnt -= 64) {
-                cp = mempcpy (cp, temp_result, 64);
+                cp = memcpy (cp, temp_result, 64);
+                cp += 64;
         }
         memcpy (cp, temp_result, cnt);
 
@@ -344,7 +350,8 @@ inline char *sha512_crypt_r(const char *key, const char *salt, size_t maxrounds_
         /* Create byte sequence S. */
         cp = s_bytes = alloca (salt_len);
         for (cnt = salt_len; cnt >= 64; cnt -= 64) {
-                cp = mempcpy (cp, temp_result, 64);
+                cp = memcpy (cp, temp_result, 64);
+                cp += 64;
         }
         memcpy (cp, temp_result, cnt);
 
@@ -385,7 +392,7 @@ inline char *sha512_crypt_r(const char *key, const char *salt, size_t maxrounds_
 
         /* Now we can construct the result string.  It consists of three
            parts. */
-        cp = __stpncpy (buffer, salt, MIN ((size_t) MAX (0, buflen), salt_len));
+        cp = stpncpy (buffer, salt, MIN ((size_t) MAX (0, buflen), salt_len));
         buflen -= MIN ((size_t) MAX (0, buflen), salt_len);
 
         b64_from_24bit (alt_result[0],  alt_result[21], alt_result[42], 4);
@@ -472,6 +479,8 @@ char *eurephia_pwd_crypt(eurephiaCTX *ctx, const char *key, const char *salt) {
 
         buffer = (char *) malloc_nullsafe(ctx, buflen);
         assert(buffer != NULL);
+        mlock(buffer, buflen);
+        mlock(&saltstr, MAX_SALT_LEN+22);
 
         // Get default max rounds for hashing
         if( maxrounds == 0 ) {
@@ -491,6 +500,7 @@ char *eurephia_pwd_crypt(eurephiaCTX *ctx, const char *key, const char *salt) {
 
                 tmp = malloc_nullsafe(ctx, saltlen+2);
                 assert(tmp != NULL);
+                mlock(tmp, saltlen+2);
                 memset(&saltstr, 0, MAX_SALT_LEN+22);
 
                 // Get default min rounds for hashing
@@ -503,7 +513,7 @@ char *eurephia_pwd_crypt(eurephiaCTX *ctx, const char *key, const char *salt) {
                         loop++;
                 } while( ((rounds < minrounds) || (rounds > maxrounds)) && (loop < 1000)) ;
 
-                if( loop > 10000 ) {
+                if( loop >= 1000 ) {
                         eurephia_log(ctx, LOG_FATAL, 0,
                                      "Could not get a valid random number for hashing after %i rounds", 1000);
                         return NULL;
@@ -520,6 +530,7 @@ char *eurephia_pwd_crypt(eurephiaCTX *ctx, const char *key, const char *salt) {
                 strncpy(saltstr, saltinfo, strlen(saltinfo));
                 strncat(saltstr, tmp, saltlen - strlen(saltinfo));
                 memset(tmp, 0, saltlen+2);
+                munlock(tmp, saltlen+2);
                 free_nullsafe(ctx, tmp);
         } else {
                 // If we have a salt, use it
@@ -527,7 +538,10 @@ char *eurephia_pwd_crypt(eurephiaCTX *ctx, const char *key, const char *salt) {
         }
         // For some reason, if not strdup()ing 'buffer' and returning buffer it causes a memory leak
         result = strdup_nullsafe(sha512_crypt_r(key, saltstr, maxrounds, buffer, buflen));
+        memset(buffer, 0, buflen);
+        munlock(buffer, buflen);
         free_nullsafe(NULL, buffer);
+        munlock(&saltstr, MAX_SALT_LEN+22);
         return result;
 }
 
@@ -560,8 +574,12 @@ char *eurephia_quick_hash(const char *salt, const char *pwd) {
         } else {
                 tmp = strdup_nullsafe(pwd);
         }
+        mlock(tmp, strlen_nullsafe(tmp));
+
         // Generate SHA512 hash of password
+        mlock(&sha, sizeof(SHA512Context));
         memset(&sha, 0, sizeof(SHA512Context));
+        mlock(&sha_res, sizeof(sha_res));
         memset(&sha_res, 0, sizeof(sha_res));
         SHA512Init(&sha);
         SHA512Update(&sha, tmp, len);
@@ -580,6 +598,12 @@ char *eurephia_quick_hash(const char *salt, const char *pwd) {
         // Cleanup - remove hash data from memory
         memset(&sha, 0, sizeof(SHA512Context));
         memset(&sha_res, 0, sizeof(sha_res));
+        munlock(&sha, sizeof(SHA512Context));
+        munlock(&sha_res, sizeof(sha_res));
+
+        len = strlen_nullsafe(tmp);
+        memset(tmp, 0, len);
+        munlock(tmp, len);
         free_nullsafe(NULL, tmp);
 
         return ret;
