@@ -389,7 +389,7 @@ int eDBblacklist_check(eurephiaCTX *ctx, const int type, const char *val)
                 // Update attempt information
                 update_attempts(ctx, blid);
         } else {
-                eurephia_log(ctx, LOG_FATAL, 0, "Quering openvpn_blacklist for blacklisted %s failed",
+                eurephia_log(ctx, LOG_FATAL, 0, "Querying openvpn_blacklist for blacklisted %s failed",
                              eDBattempt_types[type].descr);
         }
 
@@ -427,7 +427,7 @@ int eDBblacklist_check(eurephiaCTX *ctx, const int type, const char *val)
                         free_nullsafe(atpid);
                 } else {
                         eurephia_log(ctx, LOG_CRITICAL, 0,
-                                     "Quering openvpn_attempts for blacklisted %s failed",
+                                     "Querying openvpn_attempts for blacklisted %s failed",
                                      eDBattempt_types[type].descr);
                 }
                 free_nullsafe(atpr);
