@@ -1,6 +1,6 @@
 /* eurephiadb_driver.h  --  API provided by the database driver
  *
- *  GPLv2 only - Copyright (C) 2008 - 2010
+ *  GPLv2 only - Copyright (C) 2008 - 2012
  *               David Sommerseth <dazo@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or
@@ -28,6 +28,7 @@
  *
  */
 
+#include <eurephia_driverapi.h>
 #include <eurephia_context.h>
 #include <eurephiadb_session_struct.h>
 #include <eurephia_values_struct.h>
@@ -52,14 +53,12 @@
 #define USERINFO_attempts   0x08 /**< flag for extracting information from attempts log */
 #define USERINFO_blacklist  0x10 /**< flag for extracting information from blacklist log */
 
-#ifndef DRIVER_MODE
-
 /**
  * Mandatory function. Retrieves driver version information
  *
  * @return Returns string (const char *) containing driver version information.
  */
-const char *(*eDB_DriverVersion) (void);
+const char * EUREPHIA_DRIVERAPI_FUNC(eDB_DriverVersion)(void);
 
 
 /**
@@ -67,7 +66,7 @@ const char *(*eDB_DriverVersion) (void);
  *
  * @return Returns integer value with API level supported by driver.
  */
-int (*eDB_DriverAPIVersion) (void);
+int EUREPHIA_DRIVERAPI_FUNC(eDB_DriverAPIVersion)(void);
 
 /*
  *    functions which needs to exists in the API level 1
@@ -84,7 +83,7 @@ int (*eDB_DriverAPIVersion) (void);
  *
  * @return Returns 1 on success, otherwise 0.
  */
-int (*eDBconnect) (eurephiaCTX *ctx, const int argc, const char **argv);
+int EUREPHIA_DRIVERAPI_FUNC(eDBconnect)(eurephiaCTX *ctx, const int argc, const char **argv);
 
 /**
  * Disconnects from a database
@@ -93,7 +92,7 @@ int (*eDBconnect) (eurephiaCTX *ctx, const int argc, const char **argv);
  * @param ctx eurephiaCTX - context with the database connection to disconnect from.
  *
  */
-void (*eDBdisconnect) (eurephiaCTX *ctx);
+void EUREPHIA_DRIVERAPI_FUNC(eDBdisconnect) (eurephiaCTX *ctx);
 
 /**
  * Authenticates a certificate against the database.
@@ -110,8 +109,9 @@ void (*eDBdisconnect) (eurephiaCTX *ctx);
  * @return Returns certid (certificate ID) on success.  0 is returned if certificate is not found,
  *         or -1 if the certificate is blacklisted.
  */
-int (*eDBauth_TLS) (eurephiaCTX *ctx, const char *org, const char *cname, const char *email,
-                    const char *digest, const unsigned int depth);
+int EUREPHIA_DRIVERAPI_FUNC(eDBauth_TLS)(eurephiaCTX *ctx, const char *org, const char *cname,
+					 const char *email,
+					 const char *digest, const unsigned int depth);
 
 /**
  * Authenticates a client against the database, with users certificate ID, username and password.
@@ -125,7 +125,8 @@ int (*eDBauth_TLS) (eurephiaCTX *ctx, const char *org, const char *cname, const
  * @return Returns uicid (user-certs ID) to the user on success.  0 if user account is not
  *         found and -1 on authentication failure.
  */
-int (*eDBauth_user) (eurephiaCTX *ctx, const int certid, const char *username, const char *passwd);
+int EUREPHIA_DRIVERAPI_FUNC(eDBauth_user)(eurephiaCTX *ctx, const int certid,
+					  const char *username, const char *passwd);
 
 
 /**
@@ -138,7 +139,7 @@ int (*eDBauth_user) (eurephiaCTX *ctx, const int certid, const char *username, c
  *
  * @return Returns uid of user on success, 0 if user account is not found, otherwise -1 on errors.
  */
-int (*eDBget_uid) (eurephiaCTX *ctx, const int certid, const char *username);
+int EUREPHIA_DRIVERAPI_FUNC(eDBget_uid)(eurephiaCTX *ctx, const int certid, const char *username);
 
 
 /**
@@ -154,7 +155,7 @@ int (*eDBget_uid) (eurephiaCTX *ctx, const int certid, const char *username);
  *
  * @see attempt_IPADDR, attempt_CERTIFICATE, attempt_USERNAME
  */
-int (*eDBblacklist_check) (eurephiaCTX *ctx, const int type, const char *val);
+int EUREPHIA_DRIVERAPI_FUNC(eDBblacklist_check)(eurephiaCTX *ctx, const int type, const char *val);
 
 
 /**
@@ -170,7 +171,7 @@ int (*eDBblacklist_check) (eurephiaCTX *ctx, const int type, const char *val);
  *
  * @see ATTEMPT_REGISTER, ATTEMPT_RESET, attempt_IPADDR, attempt_CERTIFICATE, attempt_USERNAME
  */
-void (*eDBregister_attempt) (eurephiaCTX *ctx, int type, int mode, const char *value);
+void EUREPHIA_DRIVERAPI_FUNC(eDBregister_attempt)(eurephiaCTX *ctx, int type, int mode, const char *value);
 
 
 /**
@@ -190,9 +191,10 @@ void (*eDBregister_attempt) (eurephiaCTX *ctx, int type, int mode, const char *v
  *
  * @return Returns 1 on success, otherwise 0.
  */
-int (*eDBregister_login) (eurephiaCTX *ctx, eurephiaSESSION *skey, const int certid, const int uid,
-                          const char *proto, const char *remipaddr, const char *remport,
-                          const char *vpnipaddr, const char *vpnipmask);
+int EUREPHIA_DRIVERAPI_FUNC(eDBregister_login)(eurephiaCTX *ctx, eurephiaSESSION *skey,
+					       const int certid, const int uid,
+					       const char *proto, const char *remipaddr, const char *remport,
+					       const char *vpnipaddr, const char *vpnipmask);
 
 
 /**
@@ -206,9 +208,29 @@ int (*eDBregister_login) (eurephiaCTX *ctx, eurephiaSESSION *skey, const int cer
  *
  * @return Returns 1 on success, otherwise 0.
  */
-int (*eDBregister_vpnmacaddr) (eurephiaCTX *ctx, eurephiaSESSION *session, const char *macaddr);
+int EUREPHIA_DRIVERAPI_FUNC(eDBregister_vpnmacaddr)(eurephiaCTX *ctx, eurephiaSESSION *session,
+						    const char *macaddr);
 
 
+/**
+ *  Registers the VPN clients address.  This function is called when
+ *  OpenVPN does the OPENVPN_PLUGIN_LEARN_ADDRESS call to the eurephia-auth plug-in.
+ *  In TAP mode the clients VPN MAC and IP address are stored, in TUN mode the VPN IP
+ *  address is stored.
+ *
+ * @version API version level 3
+ * @param ctx        eurephiaCTX
+ * @param session    eurephiaSESSION of the user
+ * @param macaddr    String (char *) containing the MAC address of the clients interface.
+ * @param vpnip4addr String (char *) containing the IPv4 address of the clients interface.
+ * @param vpnip6addr String (char *) containing the IPv6 address of the clients interface.
+ *
+ * @return Returns 1 on success, otherwise 0.
+ */
+int EUREPHIA_DRIVERAPI_FUNC(eDBregister_vpnclientaddr)(eurephiaCTX *ctx, eurephiaSESSION *session,
+                                                       const char * macaddr,
+                                                       const char * vpnip4addr, const char *vpnip6addr);
+
 /**
  * Registers when a user logged out.  It will then add some information about the session to the
  * eurephia lastlog.
@@ -222,8 +244,9 @@ int (*eDBregister_vpnmacaddr) (eurephiaCTX *ctx, eurephiaSESSION *session, const
  *
  * @return Returns 1 on success, otherwise 0.
  */
-int (*eDBregister_logout) (eurephiaCTX *ctx, eurephiaSESSION *skey,
-                           const char *bytes_sent, const char *bytes_received, const char *duration);
+int EUREPHIA_DRIVERAPI_FUNC(eDBregister_logout)(eurephiaCTX *ctx, eurephiaSESSION *skey,
+						const char *bytes_sent, const char *bytes_received,
+						const char *duration);
 
 
 /* firewall functions */
@@ -238,7 +261,7 @@ int (*eDBregister_logout) (eurephiaCTX *ctx, eurephiaSESSION *skey,
  * @return Returns a char pointer to a buffer with the name of the firewall profile. This buffer
  *         must be freed when no longer needed.
  */
-char *(*eDBget_firewall_profile) (eurephiaCTX *ctx, eurephiaSESSION *session);
+char * EUREPHIA_DRIVERAPI_FUNC(eDBget_firewall_profile) (eurephiaCTX *ctx, eurephiaSESSION *session);
 
 /**
  * Retrieve a list of IP addresses found in the IP address blacklist table.
@@ -249,7 +272,7 @@ char *(*eDBget_firewall_profile) (eurephiaCTX *ctx, eurephiaSESSION *session);
  * @return Returns an eurephiaVALUES chain with all blacklisted IP addresses on success, otherwise
  *         NULL is returned
  */
-eurephiaVALUES *(*eDBget_blacklisted_ip) (eurephiaCTX *ctx);
+eurephiaVALUES * EUREPHIA_DRIVERAPI_FUNC(eDBget_blacklisted_ip)(eurephiaCTX *ctx);
 
 /* The following functions is also declared in eurephia_session_values.c - for local internal usage. */
 /**
@@ -264,7 +287,8 @@ eurephiaVALUES *(*eDBget_blacklisted_ip) (eurephiaCTX *ctx);
  * @return Returns the unique session key string (char *) on success, otherwise NULL.  The session key
  *         string must be freed when no longer needed.
  */
-char *(*eDBget_sessionkey_seed) (eurephiaCTX *ctx, sessionType type, const char *sessionseed);
+char * EUREPHIA_DRIVERAPI_FUNC(eDBget_sessionkey_seed)(eurephiaCTX *ctx, sessionType type,
+						      const char *sessionseed);
 
 
 /**
@@ -278,7 +302,7 @@ char *(*eDBget_sessionkey_seed) (eurephiaCTX *ctx, sessionType type, const char
  * @return Returns the unique session key string (char *) on success, otherwise NULL.  The session key
  *         string must be freed when no longer needed.
  */
-char *(*eDBget_sessionkey_macaddr) (eurephiaCTX *ctx, const char *macaddr);
+char * EUREPHIA_DRIVERAPI_FUNC(eDBget_sessionkey_macaddr)(eurephiaCTX *ctx, const char *macaddr);
 
 
 /**
@@ -291,7 +315,7 @@ char *(*eDBget_sessionkey_macaddr) (eurephiaCTX *ctx, const char *macaddr);
  *
  * @return Returns 1 if the session key is unique and not been used earlier.
  */
-int (*eDBcheck_sessionkey_uniqueness) (eurephiaCTX *ctx, const char *seskey);
+int EUREPHIA_DRIVERAPI_FUNC(eDBcheck_sessionkey_uniqueness)(eurephiaCTX *ctx, const char *seskey);
 
 
 /**
@@ -304,7 +328,7 @@ int (*eDBcheck_sessionkey_uniqueness) (eurephiaCTX *ctx, const char *seskey);
  *
  * @return Returns 1 on success, otherwise 0
  */
-int (*eDBregister_sessionkey) (eurephiaCTX *ctx, const char *seed, const char *seskey);
+int EUREPHIA_DRIVERAPI_FUNC(eDBregister_sessionkey)(eurephiaCTX *ctx, const char *seed, const char *seskey);
 
 
 /**
@@ -318,7 +342,7 @@ int (*eDBregister_sessionkey) (eurephiaCTX *ctx, const char *seed, const char *s
  * @return Returns a pointer to an eurephiaVALUES pointer chain with all the variables available
  *         for the given session.  On errors, it will return an empty eurephiaVALUES chain.
  */
-eurephiaVALUES *(*eDBload_sessiondata) (eurephiaCTX *ctx, const char *sesskey);
+eurephiaVALUES * EUREPHIA_DRIVERAPI_FUNC(eDBload_sessiondata)(eurephiaCTX *ctx, const char *sesskey);
 
 
 /**
@@ -331,7 +355,7 @@ eurephiaVALUES *(*eDBload_sessiondata) (eurephiaCTX *ctx, const char *sesskey);
  *
  * @return Returns 1 on success, otherwise 0.
  */
-int (*eDBdestroy_session) (eurephiaCTX *ctx, eurephiaSESSION *session);
+int EUREPHIA_DRIVERAPI_FUNC(eDBdestroy_session)(eurephiaCTX *ctx, eurephiaSESSION *session);
 
 
 /**
@@ -348,8 +372,9 @@ int (*eDBdestroy_session) (eurephiaCTX *ctx, eurephiaSESSION *session);
  * @return Returns 1 on success, otherwise 0
  * @see SESSVAL_NEW, SESSVAL_UPDATE, SESSVAL_DELETE
  */
-int (*eDBstore_session_value)(eurephiaCTX *ctx, eurephiaSESSION *session, int mode,
-                              const char *key, const char *val);
+int EUREPHIA_DRIVERAPI_FUNC(eDBstore_session_value)(eurephiaCTX *ctx, eurephiaSESSION *session, int mode,
+						    const char *key, const char *val);
+
 
 /*
  *    functions which needs to exists in the API level 2
@@ -403,7 +428,7 @@ int (*eDBstore_session_value)(eurephiaCTX *ctx, eurephiaSESSION *session, int mo
  *         user will be returned in the details part of the ResultMsg.
  *
  */
-xmlDoc *(*eDBadminAuthenticate) (eurephiaCTX *ctx, xmlDoc *qryxml);
+xmlDoc * EUREPHIA_DRIVERAPI_FUNC(eDBadminAuthenticate)(eurephiaCTX *ctx, xmlDoc *qryxml);
 
 
 /**
@@ -427,7 +452,7 @@ xmlDoc *(*eDBadminAuthenticate) (eurephiaCTX *ctx, xmlDoc *qryxml);
  * @return Returns a valid XML document with information about the operation, or NULL on fatal errors.
  * @see eurephiaXML_CreateDoc()
  */
-xmlDoc *(*eDBadminConfiguration)(eurephiaCTX *ctx, xmlDoc *cfgxml);
+xmlDoc * EUREPHIA_DRIVERAPI_FUNC(eDBadminConfiguration)(eurephiaCTX *ctx, xmlDoc *cfgxml);
 
 
 /**
@@ -471,7 +496,7 @@ xmlDoc *(*eDBadminConfiguration)(eurephiaCTX *ctx, xmlDoc *cfgxml);
  *
  * @see eurephiaXML_CreateDoc(), eurephiaXML_ParseResultMsg(), eurephiaXML_getRoot()
  */
-xmlDoc *(*eDBadminUserAccount) (eurephiaCTX *ctx, xmlDoc *qryxml);
+xmlDoc * EUREPHIA_DRIVERAPI_FUNC(eDBadminUserAccount)(eurephiaCTX *ctx, xmlDoc *qryxml);
 
 
 /**
@@ -521,7 +546,7 @@ xmlDoc *(*eDBadminUserAccount) (eurephiaCTX *ctx, xmlDoc *qryxml);
  *
  * @see eurephiaXML_CreateDoc(), eurephiaXML_ParseResultMsg(), eurephiaXML_getRoot()
  */
-xmlDoc *(*eDBadminCertificate) (eurephiaCTX *ctx, xmlDoc *qryxml);
+xmlDoc * EUREPHIA_DRIVERAPI_FUNC(eDBadminCertificate)(eurephiaCTX *ctx, xmlDoc *qryxml);
 
 
 /**
@@ -550,7 +575,7 @@ xmlDoc *(*eDBadminCertificate) (eurephiaCTX *ctx, xmlDoc *qryxml);
  *         with an error message will be returned or NULL.
  * @see eurephiaXML_CreateDoc(), eurephiaXML_getRoot()
  */
-xmlDoc *(*eDBadminUserCertsLink) (eurephiaCTX *ctx, xmlDoc *usrcrt_xml);
+xmlDoc * EUREPHIA_DRIVERAPI_FUNC(eDBadminUserCertsLink)(eurephiaCTX *ctx, xmlDoc *usrcrt_xml);
 
 
 /**
@@ -597,7 +622,7 @@ xmlDoc *(*eDBadminUserCertsLink) (eurephiaCTX *ctx, xmlDoc *usrcrt_xml);
  *         On fatal errors NULL is returned.
  * @see eurephiaXML_CreateDoc(), eurephiaXML_getRoot()
  */
-xmlDoc *(*eDBadminAccessLevel) (eurephiaCTX *ctx, xmlDoc *qryxml);
+xmlDoc * EUREPHIA_DRIVERAPI_FUNC(eDBadminAccessLevel)(eurephiaCTX *ctx, xmlDoc *qryxml);
 
 
 /**
@@ -624,7 +649,7 @@ xmlDoc *(*eDBadminAccessLevel) (eurephiaCTX *ctx, xmlDoc *qryxml);
  *         NULL or an XML error document is returned.
  * @see eurephiaXML_CreateDoc(), eurephiaXML_getRoot()
  */
-xmlDoc *(*eDBadminFirewallProfiles) (eurephiaCTX *ctx, xmlDoc *xmlqry);
+xmlDoc * EUREPHIA_DRIVERAPI_FUNC(eDBadminFirewallProfiles)(eurephiaCTX *ctx, xmlDoc *xmlqry);
 
 
 /**
@@ -650,7 +675,8 @@ xmlDoc *(*eDBadminFirewallProfiles) (eurephiaCTX *ctx, xmlDoc *xmlqry);
  * @return Returns a valid XML document on success, otherwise NULL
  * @see eurephiaXML_CreateDoc(), eurephiaXML_getRoot()
  */
-xmlDoc *(*eDBadminGetLastlog) (eurephiaCTX *ctx, xmlDoc *srch_xml, const char *sortkeys);
+xmlDoc * EUREPHIA_DRIVERAPI_FUNC(eDBadminGetLastlog)(eurephiaCTX *ctx, xmlDoc *srch_xml,
+						     const char *sortkeys);
 
 
 /**
@@ -676,7 +702,7 @@ xmlDoc *(*eDBadminGetLastlog) (eurephiaCTX *ctx, xmlDoc *srch_xml, const char *s
  *         with an error message.
  * @see eurephiaXML_CreateDoc(), eurephiaXML_getRoot()
  */
-xmlDoc *(*eDBadminAttemptsLog) (eurephiaCTX *ctx, xmlDoc *qryxml);
+xmlDoc * EUREPHIA_DRIVERAPI_FUNC(eDBadminAttemptsLog)(eurephiaCTX *ctx, xmlDoc *qryxml);
 
 
 /**
@@ -703,9 +729,7 @@ xmlDoc *(*eDBadminAttemptsLog) (eurephiaCTX *ctx, xmlDoc *qryxml);
  *         with an error message.
  * @see eurephiaXML_CreateDoc(), eurephiaXML_getRoot()
  */
-xmlDoc *(*eDBadminBlacklist) (eurephiaCTX *ctx, xmlDoc *qryxml);
+xmlDoc * EUREPHIA_DRIVERAPI_FUNC(eDBadminBlacklist)(eurephiaCTX *ctx, xmlDoc *qryxml);
 #endif /* HAVE_LIBXML2 */
 
-#endif /* !DRIVER_MODE */
-
 #endif /* !EUREPHIADB_DRIVER_H_ */
