@@ -1,6 +1,6 @@
 /* sqlite.c  --  Generic functions to simplify SQLite3 queries
  *
- *  GPLv2 only - Copyright (C) 2008 - 2010
+ *  GPLv2 only - Copyright (C) 2008 - 2012
  *               David Sommerseth <dazo@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or
@@ -32,6 +32,7 @@
 #include <stdlib.h>
 #include <assert.h>
 #include <sqlite3.h>
+#include <time.h>
 
 #ifdef HAVE_LIBXML2
 # include <libxml/tree.h>
@@ -54,6 +55,23 @@
  * @}
  */
 
+/**
+ * Internal function for setting the error fields.
+ */
+static void _sqlite_set_error(dbresult *dbres, ErrorSeverity sev, const char *query, const char *fmt, ...)
+{
+        char errbuf[4098];
+        va_list ap;
+
+        va_start(ap, fmt);
+        memset(&errbuf, 0, 4098);
+        vsnprintf(errbuf, 4096, fmt, ap);
+        va_end(ap);
+
+        dbres->errSeverity = sev;
+        dbres->errMsg = strdup(errbuf);
+        dbres->query = strdup(query);
+}
 
 /**
  * Internal function.  Free up a dbresult structure.  This function is available via the
@@ -102,6 +120,11 @@ void _sqlite_free_results(dbresult *inres)
                 free_nullsafe(NULL, hdr->name);
                 free_nullsafe(NULL, hdr);
         }
+
+        // Remove error messages and copy of the SQL query
+        inres->status = dbEMPTY;
+        free_nullsafe(NULL, inres->errMsg);
+        free_nullsafe(NULL, inres->query);
         free_nullsafe(NULL, inres);
 }
 
@@ -221,6 +244,134 @@ static int _cb_parse_result(void *resultptr, int argc, char **argv, char **colNa
 }
 
 
+/**
+ * Simple date/time function which converts UTC to local time
+ *
+ * @param ctx SQLite database connection context
+ * @param argc Number of arguments available
+ * @param argv Argument list
+ *
+ */
+void _sqlitefnc_localdatetime(sqlite3_context *context, int argc, sqlite3_value **argv)
+{
+        struct tm tm, loctm;
+        time_t t;
+        char buf[255];
+
+        assert( argc == 1);
+
+        switch(sqlite3_value_type(argv[0])) {
+        case SQLITE_NULL:
+                /* NULL in is NULL out */
+                sqlite3_result_null(context);
+                break;
+
+        case SQLITE_TEXT:
+                memset(&tm, 0, sizeof(struct tm));
+                memset(&t, 0, sizeof(time_t));
+                memset(&buf, 0, sizeof(buf));
+
+                /* Convert the input datetime string to time_t format, expect it to be UTC/GMT */
+                strptime((const char *)sqlite3_value_text(argv[0]), "%Y-%m-%d %H:%M:%S", &tm);
+                t = timegm(&tm);
+
+                /* Convert from UTC/GMT to localtime and format the new output string */
+                localtime_r(&t, &loctm);
+                strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M:%S", &loctm);
+                sqlite3_result_text(context, buf, strlen(buf), SQLITE_TRANSIENT);
+                break;
+        }
+}
+
+
+/**
+ * Initialise the SQLite database with additional functions usable via SQL
+ *
+ * @param ctx eurephia context with the opened database
+ *
+ * @return Returns dbSUCCESS on success, otherwise dbERROR.
+ */
+int sqlite_init_functions(eurephiaCTX *ctx)
+{
+        int rc;
+
+        /* locdt(TIMESTAMP): datetime function which converts UTC/GMT time stamps into
+         *                   the localtime zone.  Expects the format to be 'YYYY-MM-DD HH24:MI:SS'.
+         *                   Used by the administration functions to return the correct local time
+         */
+        rc = sqlite3_create_function(ctx->dbc->dbhandle, "locdt", 1, SQLITE_ANY,
+                                     NULL, &_sqlitefnc_localdatetime, NULL, NULL);
+        if( rc != SQLITE_OK ) {
+                eurephia_log(ctx, LOG_PANIC, 0, "Failed to register local date/time function (%i)", rc);
+                return dbERROR;
+        }
+
+        return dbSUCCESS;
+}
+
+
+/**
+ *  Logs sqlite error messages via the eurephia log interface and returns an XML node with details
+ *  (This function is only included if libxml2 is available)
+ *
+ *  @param eurephiaCTX*  eurephia context pointer, where logging will be performed
+ *  @param dbresult*     Pointer to the database result with the error message
+ *
+ */
+void sqlite_log_error(eurephiaCTX *ctx, dbresult *dbres) {
+        if( dbres == NULL ) {
+                eurephia_log(ctx, LOG_CRITICAL, 2, "Unknown error (NULL result)");
+                return;
+        }
+
+        if( dbres->status != dbSUCCESS ) {
+                eurephia_log(ctx, LOG_ERROR, 4, "SQL Error: %s", dbres->errMsg);
+        }
+        DEBUG(ctx, 33, "SQL Query: %s", dbres->query);
+}
+
+
+/**
+ *  Logs sqlite error messages via the eurephia log interface and returns an XML node with details
+ *  (This function is only included if libxml2 is available)
+ *
+ *  @param eurephiaCTX*  eurephia context pointer, where logging will be performed
+ *  @param dbresult*     Pointer to the database result with the error message
+ *
+ *  @return An xmlNode pointer with more details of the error.
+ */
+#ifdef HAVE_LIBXML2
+xmlNode *sqlite_log_error_xml(eurephiaCTX *ctx, dbresult *dbres ) {
+        xmlNode *ret = NULL;
+
+        sqlite_log_error(ctx, dbres);
+        if( dbres == NULL ) {
+                return NULL;
+        }
+
+        ret = xmlNewNode(NULL, (xmlChar *) "SQLError");
+        if( ret != NULL ) {
+                xmlNode *err_n = NULL;
+                xmlChar *errstr = NULL;
+                const xmlChar *ErrorSeverity_str[] = {
+                        (xmlChar *) "Warning",
+                        (xmlChar *) "Error",
+                        (xmlChar *) "Critical",
+                        (xmlChar *) "PANIC",
+                        NULL
+                };
+
+                xmlNewProp(ret, (xmlChar *) "driver", (xmlChar *) "edb-sqlite.so");
+                errstr = xmlCharStrdup(dbres->errMsg);
+                err_n = xmlNewTextChild(ret, NULL, (xmlChar *) "ErrorMessage", errstr);
+                xmlNewProp(err_n, (xmlChar *) "severity", ErrorSeverity_str[dbres->errSeverity]);
+                free_nullsafe(NULL, errstr);
+        }
+        return ret;
+}
+#endif
+
+
 /**
  * Main function for query a SQLite3 database.
  *
@@ -236,35 +387,32 @@ dbresult *sqlite_query(eurephiaCTX *ctx, const char *fmt, ... ) {
         eDBconn *dbc = ctx->dbc;
         dbresult *dbres = NULL;
 
-        if( ctx->dbc == NULL ) {
-                eurephia_log(ctx, LOG_PANIC, 0, "No open database connection to perfom SQL query to");
-                return NULL;
-        }
-
-        if( ctx->context_type == ECTX_NO_PRIVILEGES ) {
-                eurephia_log(ctx, LOG_ERROR, 0, "Database query attempted from wrong context");
-                return NULL;
-        }
-        
-        // prepare a new (global) result set ... 
-        // do not delete the old ones, since we return this "global"
-        // result as a new individual result
+        // prepare a new result set ...
         dbres = malloc_nullsafe(ctx, sizeof(dbresult)+2);
+        dbres->status = dbEMPTY;
         dbres->num_tuples = 0;
-        
+
         // prepare SQL query
         va_start(ap, fmt);
         sql = sqlite3_vmprintf(fmt, ap);
         va_end(ap);
 
+        if( ctx->dbc == NULL ) {
+                _sqlite_set_error(dbres, sevPANIC, sql, "No open database connection to perfom SQL query to");
+                goto exit;
+        }
+
+        if( ctx->context_type == ECTX_NO_PRIVILEGES ) {
+                _sqlite_set_error(dbres, sevCRITICAL, sql, "Database query attempted from wrong context");
+                goto exit;
+        }
+
         DEBUG(ctx, 25, "Doing SQL Query: %s", sql);
         rc = sqlite3_exec( (sqlite3 *) dbc->dbhandle, sql, _cb_parse_result, dbres, &errMsg );
         if( rc != SQLITE_OK ) {
-                eurephia_log(ctx, LOG_ERROR, 0, "SQL Error: %s", errMsg);
-                sqlite3_free(sql); sql = NULL;
+                _sqlite_set_error(dbres, (dbres->num_tuples > 0 ? sevWARNING : sevERROR), sql, "%s", errMsg);
                 sqlite3_free(errMsg); errMsg = NULL;
-                free_nullsafe(ctx, dbres);
-                return NULL;
+                goto exit;
         }
 
         if( strcasestr(sql, "INSERT INTO") != 0) {
@@ -275,8 +423,10 @@ dbresult *sqlite_query(eurephiaCTX *ctx, const char *fmt, ... ) {
                 dbres->affected_rows = sqlite3_changes((sqlite3 *) dbc->dbhandle);
         }
 
+        dbres->status = dbSUCCESS;
         dbres->srch_tuples = dbres->tuples;
         dbres->srch_headerrec = dbres->headerrec;
+ exit:
         sqlite3_free(sql); sql = NULL;
 
         return dbres;
@@ -671,16 +821,18 @@ int main(int argc, char **argv) {
 
         ctx = malloc_nullsafe(NULL, sizeof(eurephiaCTX)+2);
         ctx->dbc = malloc_nullsafe(NULL, sizeof(eDBconn)+2);
-        
+        eurephia_log_init(ctx, "sqlitedbg", "stderr:", 10);
+
         rc = sqlite3_open(argv[1], (void *) &ctx->dbc->dbhandle);
         if( rc ) {
                 fprintf(stderr, "Could not open db\n");
                 return 1;
         }
-        
-        ctx->log = stderr;
-        ctx->loglevel = 5;
-        
+
+        if( sqlite_init_functions(ctx) != dbSUCCESS ) {
+                fprintf(stderr, "Failed to register functions\n");
+                return 1;
+        }
 
         res = sqlite_query(ctx, argv[2]);
         if( res != NULL ) {
@@ -689,6 +841,7 @@ int main(int argc, char **argv) {
         }
         sqlite3_close(ctx->dbc->dbhandle);
         free(ctx->dbc);
+        eurephia_log_close(ctx);
         free(ctx);
         return 0;
 }
