@@ -1,7 +1,7 @@
 /* certificates.c  --  eurephiadm command: certs 
  *                     Certificate management
  *
- *  GPLv2 only - Copyright (C) 2008 -2010
+ *  GPLv2 only - Copyright (C) 2008 - 2012
  *               David Sommerseth <dazo@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or
@@ -218,6 +218,7 @@ int register_certificate(eurephiaCTX *ctx, int depth, const char *digest,
                         certid = 0;
                 }
                 free_nullsafe(ctx, res);
+                xmlFreeDoc(res_xml);
         }
         xmlFreeDoc(cert_xml);
         free_nullsafe(ctx, cname_cp);
@@ -275,20 +276,23 @@ int add_cert(eurephiaCTX *ctx, eurephiaSESSION *sess, eurephiaVALUES *cfg, int a
                         }
                         if( chk == 0 )  {
                                 fprintf(stderr, "%s: Certificate depth must be a number\n", MODULE);
-                                return 1;
+                                rc = 1;
+                                goto exit;
                         }
 
                         depth = atoi_nullsafe(optargs[0]);
                         if( (depth < 0) || (depth > 99) ) {
                                 fprintf(stderr, "%s: Certificate depth must be between 0-99\n", MODULE);
-                                return 1;
+                                rc = 1;
+                                goto exit;
                         }
                         break;
 
                 case 'D': // Certificate digest
                         if( strlen_nullsafe(optargs[0]) < 59 ) {
                                 fprintf(stderr, "%s: Certificate digest is too short\n", MODULE);
-                                return 1;
+                                rc = 1;
+                                goto exit;
                         }
                         digest = strdup_nullsafe(optargs[0]);
                         break;
@@ -310,48 +314,57 @@ int add_cert(eurephiaCTX *ctx, eurephiaSESSION *sess, eurephiaVALUES *cfg, int a
                 case 'f': // Load certificate info from a certificate file
                         if( strlen_nullsafe(optargs[0]) < 1 ) {
                                 fprintf(stderr, "%s: certfile is too short\n", MODULE);
-                                return 1;
+                                rc = 1;
+                                goto exit;
                         }
                         certfile = optargs[0];
 
                         if( stat(certfile, &cert_stat) == -1 ) {
                                 fprintf(stderr, "%s: Could not access certfile: %s (%s)\n", MODULE,
                                         certfile, strerror(errno));
-                                return 1;
+                                rc = 1;
+                                goto exit;
                         }
 
                         if( cert_stat.st_size == 0 ) {
                                 fprintf(stderr, "%s: certfile '%s' is empty\n", MODULE, certfile);
-                                return 1;
+                                rc = 1;
+                                goto exit;
                         }
                         break;
 #endif
                 case 'h':
                         display_certs_help('A');
-                        break;
+                        rc = 0;
+                        goto exit;
+
                 default:
-                        return 2;
+                        rc = 1;
+                        goto exit;
                 }
         }
 
         // Sanity check of input parameters
         if( depth < 0 ) {
                 fprintf(stderr, "%s: You must set certificate depth (possibly, it needs to be 0)\n", MODULE);
-                return 1;
+                rc = 1;
+                goto exit;
         }
 
         if( (certfile != NULL) && ((digest != NULL) || (cname != NULL) || (org != NULL) || (email != NULL)) ){
                 fprintf(stderr,
                         "%s: You cannot combine --certfile with --depth, --common-name,\n"
                         "--organisation or --email\n", MODULE);
-                return 1;
+                rc = 1;
+                goto exit;
         }
 
         if( (certfile == NULL) && ((digest == NULL) || (cname == NULL) || (org == NULL) || (email == NULL)) ) {
                 fprintf(stderr,
                         "%s: You must use either --certfile or --depth, --common-name,\n"
                         "--organisation and --email\n", MODULE);
-                return 1;
+                rc = 1;
+                goto exit;
         }
 
 #ifdef HAVE_OPENSSL
@@ -415,14 +428,16 @@ int delete_cert(eurephiaCTX *ctx, eurephiaSESSION *sess, eurephiaVALUES *cfg, in
                 case 'i':
                         if( atoi_nullsafe(optargs[0]) < 1 ) {
                                 fprintf(stderr, "%s: Certificate ID (certid) must be > 0\n", MODULE);
-                                return 1;
+                                rc = 1;
+                                goto exit;
                         }
                         certid = optargs[0];
                         break;
                 case 'd': // Certificate digest
                         if( strlen_nullsafe(optargs[0]) < 59 ) {
                                 fprintf(stderr, "%s: Certificate digest is too short\n", MODULE);
-                                return 1;
+                                rc = 1;
+                                goto exit;
                         }
                         digest = optargs[0];
                         break;
@@ -437,9 +452,11 @@ int delete_cert(eurephiaCTX *ctx, eurephiaSESSION *sess, eurephiaVALUES *cfg, in
                         break;
                 case 'h':
                         display_certs_help('D');
-                        break;
+                        rc = 0;
+                        goto exit;
                 default:
-                        return 1;
+                        rc = 1;
+                        goto exit;
                 }
         }
 
@@ -448,7 +465,8 @@ int delete_cert(eurephiaCTX *ctx, eurephiaSESSION *sess, eurephiaVALUES *cfg, in
                 fprintf(stderr,
                         "%s: You must add at least one search criteria to delete a certificate\n",
                         MODULE);
-                return 1;
+                rc = 1;
+                goto exit;
         }
 
         // Create a field mapping containing our search criteria(s)
@@ -479,12 +497,14 @@ int delete_cert(eurephiaCTX *ctx, eurephiaSESSION *sess, eurephiaVALUES *cfg, in
         // Look up the certificate info and display it
         certlist = eDBadminCertificate(ctx, cert_xml);
         if( certlist == NULL ) {
-                return 0;
+                rc = 0;
+                goto exit;
         }
 
         delete_n = eurephiaXML_getRoot(ctx, certlist, "certificates", 1);
         if( atoi_nullsafe(xmlGetAttrValue(delete_n->properties, "certificates")) == 0 ) {
                 printf("%s: No certificates found\n", MODULE);
+                rc = 0;
                 goto exit;
         }
 
@@ -525,6 +545,7 @@ int delete_cert(eurephiaCTX *ctx, eurephiaSESSION *sess, eurephiaVALUES *cfg, in
                         rc = 0;
                 }
                 free_nullsafe(ctx, res);
+                xmlFreeDoc(res_xml);
         }
         xmlFreeDoc(delete_xml);
 
