@@ -1,6 +1,6 @@
 /* users.c  --  eurephiadm command - User Account Management
  *
- *  GPLv2 only - Copyright (C) 2008 - 2010
+ *  GPLv2 only - Copyright (C) 2008 - 2012
  *               David Sommerseth <dazo@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or
@@ -345,9 +345,11 @@ int show_user(eurephiaCTX *ctx, eurephiaSESSION *sess, eurephiaVALUES *cfg, int
 
                 case 'h':
                         display_users_help('s');
+                        xmlFreeDoc(srch_xml);
                         return 0;
 
                 default:
+                        xmlFreeDoc(srch_xml);
                         return 1;
                 }
         }
@@ -517,6 +519,7 @@ int account_activation(eurephiaCTX *ctx, eurephiaSESSION *sess, eurephiaVALUES *
 
                 case 'h':
                         display_users_help(actmode);
+                        xmlFreeDoc(srch_xml);
                         return 0;
 
                 case 'P':
@@ -524,6 +527,7 @@ int account_activation(eurephiaCTX *ctx, eurephiaSESSION *sess, eurephiaVALUES *
                         break;
 
                 default:
+                        xmlFreeDoc(srch_xml);
                         return 1;
                 }
         }
@@ -682,6 +686,7 @@ int account_activation(eurephiaCTX *ctx, eurephiaSESSION *sess, eurephiaVALUES *
                         // No ResultMsg document was returned, which is also wrong.
                         break;
                 }
+
                 // Delete links between certificates associated to this user account
                 xmlFreeDoc(update_xml);
                 eurephiaXML_CreateDoc(ctx, 1, "usercerts", &update_xml, &update_n);
@@ -700,10 +705,38 @@ int account_activation(eurephiaCTX *ctx, eurephiaSESSION *sess, eurephiaVALUES *
                         rc = 0;
                 } else if( res->resultType == exmlERROR ) {
                         fprintf(stderr, "%s: %s\n", MODULE, res->message);
+                        rc = 0;
                 } else {
                         fprintf(stdout, "%s: %s\n", MODULE, res->message);
                         rc = 1;
                 }
+
+                // Remove all user access levels for the user.
+                {
+                        xmlDoc *acclvl_d = NULL, *acclvl_res_xml = NULL;
+                        xmlNode *acclvl_n = NULL, *acclvl_fm = NULL;
+                        eurephiaRESULT *acclvl_res = NULL;
+
+                        eurephiaXML_CreateDoc(ctx, 1, "admin_access", &acclvl_d, &acclvl_n);
+                        xmlNewProp(acclvl_n, (xmlChar *) "mode", (xmlChar *) "revoke");
+                        acclvl_fm = xmlNewChild(acclvl_n, NULL, (xmlChar *) "fieldMapping", NULL);
+                        xmlNewProp(acclvl_fm, (xmlChar *) "table", (xmlChar *) "eurephia_adminaccess");
+                        xmlNewChild(acclvl_fm, NULL, (xmlChar *) "uid", (xmlChar *) uid_str);
+
+                        acclvl_res_xml = eDBadminAccessLevel(ctx, acclvl_d);
+                        acclvl_res = eurephiaXML_ParseResultMsg(ctx, acclvl_res_xml);
+                        if( acclvl_res->resultType == exmlERROR ) {
+                                fprintf(stderr, "%s: %s\n", MODULE, acclvl_res->message);
+                                rc = 0;
+                        } else {
+                                fprintf(stdout, "%s: %s\n", MODULE, acclvl_res->message);
+                                rc = rc ? 1 : 0;
+                        }
+                        free_nullsafe(ctx, res);
+                        free_nullsafe(ctx, acclvl_res);
+                        xmlFreeDoc(acclvl_res_xml);
+                        xmlFreeDoc(acclvl_d);
+                }
                 free_nullsafe(ctx, res);
                 xmlFreeDoc(tmp_xml);
                 break;
@@ -716,7 +749,7 @@ int account_activation(eurephiaCTX *ctx, eurephiaSESSION *sess, eurephiaVALUES *
                         rc = 0;
                 } else {
                         fprintf(stdout, "%s: %s\n", MODULE, res->message);
-                        rc = 1;
+                        rc = rc ? 1 : 0;
                 }
                 free_nullsafe(ctx, res);
         } else {
