@@ -1,6 +1,6 @@
 /* eurephia.c  --  Main functions for the eurephia authentication module
  *
- *  GPLv2 only - Copyright (C) 2008 - 2010
+ *  GPLv2 only - Copyright (C) 2008 - 2012
  *               David Sommerseth <dazo@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or
@@ -59,13 +59,11 @@
 /**
  * Initialises the eurephia OpenVPN plug-in and prepares a eurephiaCTX (context)
  *
- * @param argv  Arguments from the openvpn configuration file.  The format is:
- *              logfile loglevel eDB_driver [eurephiaDB arguments]
- *                 1       2        3          4.....
+ * @param argv  Arguments from the openvpn configuration file.
  *
  * @return returns a pointer to a eurephiaCTX context.  On failure NULL is returned.
  */
-eurephiaCTX *eurephiaInit(const char **argv)
+eurephiaCTX *eurephiaInit(const char const **argv, const char const **envp)
 {
         static struct option eurephia_opts[] = {
                 {"log-destination", required_argument, 0, 'l'},
@@ -143,9 +141,11 @@ eurephiaCTX *eurephiaInit(const char **argv)
                 // If another log file is given, process that
                 eurephia_log_init(ctx, "eurephia-auth", logfile, loglvl);
         }
+        eurephia_log(ctx, LOG_INFO, 0, "Initialising eurephia v" EUREPHIAVERSION
+                     " (David Sommerseth (C) 2008-2012 GPLv2)");
 
         // Load the database driver
-        if( (error == 0) && eDBlink_init(ctx, dbi, 1) ) {
+        if( (error == 0) && eDBlink_init(ctx, dbi, 3) ) {
                 // Connect to the database
                 if( !eDBconnect(ctx, dbargc, dbargv) ) {
                         eurephia_log(ctx, LOG_PANIC, 0, "Could not connect to the database");
@@ -187,8 +187,15 @@ eurephiaCTX *eurephiaInit(const char **argv)
         fwintf = eGet_value(ctx->dbc->config, "firewall_interface");
         if( fwintf != NULL ) {
                 if( eFW_load(ctx, fwintf) ) {
+                        char *daemon_s = GETENV_DAEMON(envp);
+                        char *logredir_s = GETENV_DAEMONLOGREDIR(envp);
+
                         eurephia_log(ctx, LOG_INFO, 0, "Loaded firewall interface: %s", fwintf);
-                        eFW_StartFirewall(ctx);
+                        eFW_StartFirewall(ctx,
+                                          (daemon_s && (daemon_s[0] == '1')),
+                                          (logredir_s && logredir_s[0] == '1'));
+                        free_nullsafe(ctx, daemon_s);
+                        free_nullsafe(ctx, logredir_s);
                 } else {
                         eurephia_log(ctx, LOG_FATAL, 0, "Loading of firewall interface failed (%s)", fwintf);
                         ctx->eurephia_fw_intf = NULL;
@@ -197,6 +204,12 @@ eurephiaCTX *eurephiaInit(const char **argv)
                 ctx->eurephia_fw_intf = NULL;
         }
 
+        // Prepare an empty disconnected list.
+        // This one is used to track all clients IP addresses and their corresponding eurephia session ID
+        // when they disconnect.  This is especially needed in TUN mode, the eurephia_learn_address()
+        // delete call with otherwise have too little information to identify the session.
+        ctx->disconnected = eCreate_value_space(ctx, 12);
+
         eurephia_log(ctx, LOG_INFO, 1, "eurephia-auth is initialised");
         return ctx;
 }
@@ -216,6 +229,11 @@ int eurephiaShutdown(eurephiaCTX *ctx)
                 return 0;
         }
 
+        if( ctx->disconnected != NULL ) {
+                eFree_values(ctx, ctx->disconnected);
+                ctx->disconnected = NULL;
+        }
+
         if( ctx->eurephia_fw_intf != NULL ) {
                 if( ctx->fwcfg != NULL ) {
                         eFW_StopFirewall(ctx);
@@ -257,7 +275,7 @@ int eurephia_tlsverify(eurephiaCTX *ctx, const char **env, const char *depth_str
         int result = 0;
         char *ipaddr;
         char *tls_digest, *tls_id;
-	unsigned int depth = atoi_nullsafe(depth_str);
+        unsigned int depth = atoi_nullsafe(depth_str);
         certinfo *ci = NULL;
 
         DEBUG(ctx, 10, "** Function call: eurephia_tlsverify(...)");
@@ -269,7 +287,9 @@ int eurephia_tlsverify(eurephiaCTX *ctx, const char **env, const char *depth_str
 
                 // If fw blacklisting is configured, also blacklist there too
                 if( (ctx->fwcfg != NULL) && (ctx->fwcfg->fwblacklist != NULL ) ) {
-                        eFW_UpdateFirewall(ctx, FWRULE_BLACKLIST, ipaddr, ctx->fwcfg->fwblacklist, NULL);
+                        eFWupdateRequest req = { .mode = fwBLACKLIST, };
+                        strncpy(req.ipaddress, ipaddr, sizeof(req.ipaddress));
+                        eFW_UpdateFirewall(ctx, &req);
                 }
                 free_nullsafe(ctx, ipaddr);
                 return 0;
@@ -341,7 +361,9 @@ int eurephia_userauth(eurephiaCTX *ctx, const char **env)
 
                 // If fw blacklisting is configured, also blacklist there too
                 if( (ctx->fwcfg != NULL) && (ctx->fwcfg->fwblacklist != NULL ) ) {
-                        eFW_UpdateFirewall(ctx, FWRULE_BLACKLIST, ipaddr, ctx->fwcfg->fwblacklist, NULL);
+                        eFWupdateRequest req = { .mode = fwBLACKLIST, };
+                        strncpy(req.ipaddress, ipaddr, sizeof(req.ipaddress));
+                        eFW_UpdateFirewall(ctx, &req);
                 }
                 free_nullsafe(ctx, ipaddr);
 
@@ -588,6 +610,12 @@ int eurephia_disconnect(eurephiaCTX *ctx, const char **env) {
                 return 0;
         }
 
+        if( ctx->tuntype == tuntype_TUN ) {
+                // Put the clients IP address into the disconnected list, to be used and freed
+                // by eurephia_learn_address() afterwards
+                eAdd_value(ctx, ctx->disconnected, vpnipaddr, session->sessionkey);
+        }
+
         ret = eDBregister_logout(ctx, session, bytes_sent, bytes_rec, duration);
         eDBfree_session(ctx, session);
         eurephia_log(ctx, LOG_INFO, 1, "User '%s' logged out", uname);
@@ -627,7 +655,7 @@ int eurephia_disconnect(eurephiaCTX *ctx, const char **env) {
  *
  * @return Returns 0 on failure and 1 on success.
  */
-int eurephia_learn_address(eurephiaCTX *ctx, const char *mode, const char *macaddr, const char **env) {
+int eurephia_learn_address(eurephiaCTX *ctx, const char *mode, const char *clientaddr, const char **env) {
         eurephiaSESSION *session = NULL;
         char *digest = NULL, *cname = NULL, *uname = NULL,
                 *vpnipaddr = NULL, *vpnipmask = NULL,
@@ -635,8 +663,8 @@ int eurephia_learn_address(eurephiaCTX *ctx, const char *mode, const char *macad
         char *fwprofile = NULL, *fwdest = NULL;
         int ret = 0, fw_enabled = 0;
 
-        DEBUG(ctx, 10, "** Function call: eurephia_learn_address(ctx, '%.10s', '%.18s', ...)",
-              mode, macaddr);
+        DEBUG(ctx, 10, "** Function call: eurephia_learn_address(ctx, '%.10s', '%.64s', ...)",
+              mode, clientaddr);
 
         // Get firewall information
         fw_enabled = (eGet_value(ctx->dbc->config, "firewall_interface") != NULL);
@@ -645,7 +673,15 @@ int eurephia_learn_address(eurephiaCTX *ctx, const char *mode, const char *macad
                 eurephia_log(ctx, LOG_CRITICAL, 0, "No firewall destination defined in the config.");
         }
 
-        if( strncmp(mode, "add", 3) == 0 ) {
+        // In TUN mode, if a client reconnects quickly after a disconnect,
+        // OpenVPN re-uses the session for this and just sends an update
+        // call to LEARN_ADDRESS.  As eurephia in TUN mode treats disconnects
+        // with an explicit 'delete' (see eurephia-auth.c for more info) and
+        // don't wait for a separate LEARN_ADDRESS delete call (which never
+        // seems to come), we treat 'update' as an ordinary 'add' in TUN mode.
+        if( (strncmp(mode, "add", 3) == 0)
+            || ((ctx->tuntype == tuntype_TUN) && (strncmp(mode, "update", 6) == 0)) ) {
+
                 // Fetch needed info
                 digest    = GETENV_TLSDIGEST(ctx, env, 0);
                 cname     = GETENV_CNAME(ctx, env);
@@ -663,13 +699,24 @@ int eurephia_learn_address(eurephiaCTX *ctx, const char *mode, const char *macad
                         goto exit;
                 }
 
-                // Update openvpn_lastlog with the active MAC address, and save it as a session variable
-                ret = eDBregister_vpnmacaddr(ctx, session, macaddr);
+                // Update openvpn_lastlog with the active VPN client addresses, and save it as a
+                // session variable.  clientaddr will contain the MAC address in TAP mode
+                ret = eDBregister_vpnclientaddr(ctx, session,
+                                                (ctx->tuntype == tuntype_TAP ? clientaddr : NULL),
+                                                vpnipaddr, NULL);
 
                 if( (fw_enabled) && (fwdest != NULL) ) {
                         fwprofile = eDBget_firewall_profile(ctx, session);
                         if( fwprofile != NULL ) {
-                                eFW_UpdateFirewall(ctx, FWRULE_ADD, macaddr, fwdest, fwprofile);
+                                eFWupdateRequest req = { .mode = fwADD, };
+                                if( ctx->tuntype == tuntype_TAP ) {
+                                        strncpy(req.macaddress, clientaddr, sizeof(req.macaddress));
+                                } else {
+                                        strncpy(req.ipaddress, vpnipaddr, sizeof(req.ipaddress));
+                                }
+                                strncpy(req.rule_destination, fwdest, sizeof(req.rule_destination));
+                                strncpy(req.goto_destination, fwprofile, sizeof(req.goto_destination));
+                                eFW_UpdateFirewall(ctx, &req);
                                 free_nullsafe(ctx, fwprofile);
                         }
                 }
@@ -677,12 +724,27 @@ int eurephia_learn_address(eurephiaCTX *ctx, const char *mode, const char *macad
 
         } else if( strncmp(mode, "delete", 6) == 0 ) {
 
-                // Load the session, based on MAC address
-                session = eDBopen_session_macaddr(ctx, macaddr);
+                // Load the session
+                if( ctx->tuntype == tuntype_TAP ) {
+                        // in TAP mode - use the MAC address
+                        session = eDBopen_session_macaddr(ctx, clientaddr);
+                } else {
+                        // in TUN mode - use the disconnected list to find the session key
+                        eurephiaVALUES *sessval = eGet_valuestruct(ctx->disconnected, clientaddr);
+                        if( sessval ) {
+                                session = eDBsession_load(ctx, sessval->val, stSESSION);
+                                if( ctx->disconnected->next != NULL ) {
+                                        ctx->disconnected = eRemove_value(ctx, ctx->disconnected,
+                                                                          sessval->evgid, sessval->evid);
+                                } else {
+                                        eClear_key_value(ctx->disconnected);
+                                }
+                        }
+                }
                 if( session == NULL ) {
                         eurephia_log(ctx, LOG_WARNING, 0,
-                                     "Could not find any session connected to this MAC address: %.18s",
-                                     macaddr);
+                                     "Could not find any session connected to the client address %s",
+                                     clientaddr);
                         ret = 0;
                         goto exit;
                 }
@@ -690,7 +752,15 @@ int eurephia_learn_address(eurephiaCTX *ctx, const char *mode, const char *macad
                 if( (fw_enabled) && (fwdest != NULL) ) {
                         fwprofile = eDBget_firewall_profile(ctx, session);
                         if( fwprofile != NULL ) {
-                                eFW_UpdateFirewall(ctx, FWRULE_DELETE, macaddr, fwdest, fwprofile);
+                                eFWupdateRequest req = { .mode = fwDELETE, };
+                                if( ctx->tuntype == tuntype_TAP) {
+                                        strncpy(req.macaddress, clientaddr, sizeof(req.macaddress));
+                                } else {
+                                        strncpy(req.ipaddress, clientaddr, sizeof(req.ipaddress));
+                                }
+                                strncpy(req.rule_destination, fwdest, sizeof(req.rule_destination));
+                                strncpy(req.goto_destination, fwprofile, sizeof(req.goto_destination));
+                                eFW_UpdateFirewall(ctx, &req);
                                 free_nullsafe(ctx, fwprofile);
                         }
                 }
@@ -707,7 +777,7 @@ int eurephia_learn_address(eurephiaCTX *ctx, const char *mode, const char *macad
         free_nullsafe(ctx, cname);
         free_nullsafe(ctx, digest);
         DEBUG(ctx, 10, "** Function result: eurephia_learn_address(ctx, '%s', '%.18s', ...) = %i",
-                     mode, macaddr, ret);
+                     mode, clientaddr, ret);
 
         return ret;
 }
